""" Custom Resource api
"""
import logging

from django.conf import settings

from core_main_app.commons import exceptions as exceptions
from core_main_app.components.template import api as template_api
from core_main_app.components.template_version_manager import (
    api as template_version_manager_api,
)
from core_main_registry_app.components.custom_resource.models import (
    CustomResource,
)
from core_main_registry_app.constants import CUSTOM_RESOURCE_TYPE
from core_main_registry_app.settings import REGISTRY_XSD_FILENAME

logger = logging.getLogger(
    "core_main_registry_app.components.custom_resource.api"
)


def parse_and_save(data, current_template):
    """Parse the data to create custom resources and save them.

    Args:
     data: dict
     current_template:
    Returns:
    """
    # Build list of custom resources from data
    custom_resources_list = get_resources_from_dict(data)
    # Save custom resources
    for custom_resource in custom_resources_list:
        custom_resource.template = current_template
        try:
            custom_resource.save()
        except Exception as exception:
            raise exceptions.ModelError(str(exception))


def get_resources_from_dict(resource_dict):
    """Get list of custom resources from dictionary

    Args:
        resource_dict:

    Returns:

    """
    # Initialize list
    custom_resources_list = list()
    # Create Custom Resources
    for key in resource_dict.keys():
        resource = resource_dict[key]
        custom_resource = CustomResource()
        custom_resources_list.append(
            _create_custom_resource(custom_resource, resource, key)
        )
    # Return list of custom resources
    return custom_resources_list


def _check_curate(custom_resource):
    """Check if we have all information from configuration file for curate.

    Args:
        custom_resource:
    Returns:
    """
    if (
        _is_custom_resource_type_resource(custom_resource)
        and "core_curate_app" in settings.INSTALLED_APPS
        and (
            custom_resource.role_choice is None
            or custom_resource.role_type is None
        )
    ):
        raise exceptions.ModelError(
            "Curate app is installed. "
            "You need url key, role choice and role type configured in configuration file."
        )


def _is_custom_resource_type_resource(custom_resource):
    """Is the custom resource a resource.

    Args:
        custom_resource:
    Returns:
    """
    return custom_resource.type == CUSTOM_RESOURCE_TYPE.RESOURCE.value


def _is_type_all(resource):
    """Check if type of  resource is 'all'.

    Args:
     resource:
    Returns:
    """
    try:
        return resource["type"] == CUSTOM_RESOURCE_TYPE.ALL.value
    except Exception:
        raise exceptions.ModelError("The configuration file is not valid.")


def _create_custom_resource(custom_resource, resource, key):
    """Create a custom resource.

    Args:
     custom_resource:
     resource:
     key:
    Returns:
    """
    if _is_type_all(resource):
        custom_resource.type = CUSTOM_RESOURCE_TYPE.ALL.value
    else:
        custom_resource.type = CUSTOM_RESOURCE_TYPE.RESOURCE.value
        custom_resource.role_choice = resource.get("role_choice", None)
        custom_resource.role_type = resource.get("role_type", None)
        custom_resource.description = resource.get("description", None)
        custom_resource.refinements = resource.get("refinements", None)
    custom_resource.name_in_schema = key
    custom_resource.title = resource.get("title", None)
    custom_resource.icon = resource.get("icon", None)
    custom_resource.icon_color = resource.get("icon_color", None)
    custom_resource.display_icon = resource.get("display_icon", None)
    custom_resource.sort = resource.get("sort", None)
    _check_curate(custom_resource)
    return custom_resource


def get_all_by_template(template):
    """Return all custom resource by template.

    Args:
         template:

    Returns: custom registry collection
    """
    return CustomResource.get_all_by_template(template)


def get_all_of_current_template(request):
    """Return all custom resource of the current template.

    Args:

    Returns: custom registry collection
    """
    return get_all_by_template(_get_current_template(request=request))


def get_current_custom_resource_type_all(request):
    """Return the custom resource of the current template with type 'all'.

    Args:
        request

    Returns: custom registry collection
    """
    custom_resources = CustomResource.get_custom_resource_by_template_and_type(
        _get_current_template(request=request), CUSTOM_RESOURCE_TYPE.ALL.value
    )
    if len(custom_resources) > 1:
        raise exceptions.ModelError(
            "Multiple custom resources with type 'all' were found."
        )
    if len(custom_resources) == 0:
        raise exceptions.DoesNotExist(
            "The custom resource with type 'all' does not exist."
        )
    return custom_resources[0]


def _get_current_template(request):
    """Get the current template.

    Args:
        request:

    Returns:
    """
    current_template_version = template_version_manager_api.get_active_global_version_manager_by_title(
        REGISTRY_XSD_FILENAME, request=request
    )
    current_template = template_api.get_by_id(
        current_template_version.current, request=request
    )
    return current_template


def get_by_current_template_and_slug(slug, request):
    """Get the custom resource by template and slug

    Args:
        slug:
        request:

    Returns:

    """
    return CustomResource.get_custom_resource_by_template_and_slug(
        _get_current_template(request=request), slug
    )


def get_by_role_for_current_template(role, request):
    """Get the custom resource by template and slug

    Args:
        role:
        request:

    Returns:

    """
    return CustomResource.get_by_role_for_current_template(
        _get_current_template(request=request), role
    )


def delete_custom_resources_by_template(template):
    """Delete all custom resources related to a template.

    Args:
        template:

    Returns:
    """
    CustomResource.delete_custom_resources_by_template(template)


def save_list(list_custom_resources, force_insert=False):
    """Save list of custom resource.

    Args:
        list_custom_resources:
        force_insert:

    Returns:
    """
    for custom_resource in list_custom_resources:
        custom_resource.save(force_insert=force_insert)


def replace_custom_resources_by_template(template, data):
    """Replace all custom resources by the one in data.
    Rollback if error.

    Args:
        template:
        data:

    Returns:
    """
    old_custom_resources = []
    try:
        # old custom resources
        old_custom_resources = list(get_all_by_template(template))

        # delete old custom resources
        delete_custom_resources_by_template(template)

        # create new custom resources
        parse_and_save(data, template)

    except Exception as exception:
        # rollback old custom resource
        save_list(old_custom_resources, True)
        raise exception



""" Admin views
"""
import logging

from django.conf import settings
from django.contrib import messages
from django.http.response import HttpResponseRedirect
from django.urls import reverse
from django.utils.html import escape as html_escape
from django.views.generic import View

from core_main_app.commons.exceptions import ModelError
from core_main_app.components.template import api as template_api
from core_main_app.components.template_version_manager import (
    api as template_version_manager_api,
)
from core_main_app.utils.json_utils import load_json_string
from core_main_app.utils.rendering import admin_render
from core_main_app.views.common.ajax import EditTemplateVersionManagerView
from core_main_registry_app.components.custom_resource import (
    api as custom_resource_api,
)
from core_main_registry_app.components.template import (
    api as template_registry_api,
)
from core_main_registry_app.views.admin.forms import UploadCustomResourcesForm

logger = logging.getLogger("core_main_registry_app.views.admin.views")


class ManageTemplatesRegistryView(View):
    """Manage Templates view."""

    template = "core_main_registry_app/admin/templates/list.html"

    def get(self, request):
        """get request

        Args:
            request:

        Returns:
        """

        templates = template_version_manager_api.get_global_version_managers(
            request=request
        )
        # get all registry current template
        registry_template = (
            template_registry_api.get_current_registry_template(request)
        )
        context = {
            "object_name": "Template",
            "available": [
                template for template in templates if not template.is_disabled
            ],
            "disabled": [
                template for template in templates if template.is_disabled
            ],
            "registry_current_template": template_version_manager_api.get_by_id(
                registry_template.version_manager_id, request
            ),
            "available_template_block": "core_main_registry_app/admin/core_main_app/templates/list/available.html",
            "MULTIPLE_SCHEMAS": settings.ALLOW_MULTIPLE_SCHEMAS,
        }

        assets = {
            "js": [
                {
                    "path": "core_main_app/common/js/templates/list/restore.js",
                    "is_raw": False,
                },
                {
                    "path": "core_main_app/common/js/templates/list/modals/disable.js",
                    "is_raw": False,
                },
                EditTemplateVersionManagerView.get_modal_js_path(),
            ],
        }
        if settings.ALLOW_MULTIPLE_SCHEMAS:
            assets["js"].extend(
                [
                    {
                        "path": "core_main_app/common/js/templates/sort.js",
                        "is_raw": True,
                    },
                    {
                        "path": "core_main_registry_app/admin/js/templates/sort.raw.js",
                        "is_raw": True,
                    },
                ]
            )

        modals = [
            "core_main_app/admin/templates/list/modals/disable.html",
            EditTemplateVersionManagerView.get_modal_html_path(),
        ]

        return admin_render(
            request,
            self.template,
            assets=assets,
            context=context,
            modals=modals,
        )


class UploadCustomResource(View):
    """Upload custom resource admin view."""

    form_class = UploadCustomResourcesForm
    template_name = "core_main_registry_app/admin/custom_registry/upload.html"
    object_name = "Custom Resources"

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.context = {}
        self.context.update({"object_name": self.object_name})

    def get(self, request, *args, **kwargs):
        self.context.update({"upload_form": self.form_class()})
        return admin_render(request, self.template_name, context=self.context)

    def post(self, request, *args, **kwargs):
        template_id = kwargs.pop("template_id")
        form = self.form_class(request.POST, request.FILES)
        self.context.update({"upload_form": form})

        if form.is_valid():
            return self._save_custom_resources(request, template_id)

        # Display error from the form
        return admin_render(request, self.template_name, context=self.context)

    def _save_custom_resources(self, request, template_id):
        """Saves a JSON Custom resource.

        Args:
            request: Request.
            template_id

        """
        try:
            # get the template
            template = template_api.get_by_id(template_id, request=request)

            # get the file from the form
            upload_file = request.FILES["json_file"].read().decode("utf-8")

            data = load_json_string(upload_file)
            custom_resource_api.replace_custom_resources_by_template(
                template, data
            )

            return HttpResponseRedirect(
                reverse("core-admin:core_main_registry_app_custom_registry")
            )
        except Exception as exception:
            self.context.update({"errors": html_escape(str(exception))})
            return admin_render(
                request, self.template_name, context=self.context
            )


class CustomRegistry(View):
    """Custom registry admin view."""

    def get(self, request, *args, **kwargs):
        """Get Custom Registry configuration page

        Args:
            request:
            *args:
            **kwargs:

        Returns:

        """
        data_context_list = []
        try:
            # get the current version
            current_registry_template = (
                template_registry_api.get_current_registry_template(request)
            )
            # for each template versions
            for version in current_registry_template.version_manager.versions:
                # get the template
                template = template_api.get_by_id(version, request=request)
                # get all the template's custom resources
                custom_resources = custom_resource_api.get_all_by_template(
                    template
                )
                # append to list for the context
                data_context_list.append(
                    {
                        "template_name": template.display_name,
                        "count": custom_resources.count(),
                        "template_id": str(template.id),
                        "is_current": current_registry_template
                        == str(template.id),
                    }
                )
        except ModelError as model_error:
            messages.add_message(request, messages.ERROR, str(model_error))

        modals = []

        assets = {
            "js": [],
        }

        context = {
            "objects": data_context_list,
            "object_name": "Custom Resources",
        }

        return admin_render(
            request,
            "core_main_registry_app/admin/custom_registry/list.html",
            modals=modals,
            assets=assets,
            context=context,
        )



""" Fixtures files for Custom Resource
"""
import json
from os.path import join, dirname, realpath

from core_main_app.components.template.models import Template
from core_main_app.utils.integration_tests.fixture_interface import (
    FixtureInterface,
)
from core_main_registry_app.components.custom_resource.models import (
    CustomResource,
)
from core_main_registry_app.constants import CUSTOM_RESOURCE_TYPE


class CustomResourceFixtures(FixtureInterface):
    """Custom Resource Fixture"""

    custom_resource = None
    template = None
    custom_resource_collection = None

    def insert_data(self):
        """Insert a set of Data.

        Returns:

        """
        self.template = CustomResourceFixtures.create_and_save_template()
        self.custom_resource = CustomResourceFixtures.create_custom_resource(
            template=self.template, slug="test"
        )
        self.custom_resource_collection = [self.custom_resource]

    @staticmethod
    def create_and_save_template():
        """create_and_save_template

        Returns:

        """
        template = Template(
            filename="Schema",
            content="<schema xmlns='http://www.w3.org/2001/XMLSchema'></schema>",
            _hash="",
        )
        template.save()
        return template

    @staticmethod
    def create_custom_resource(
        template=None,
        name_in_schema="",
        title="title",
        description="",
        slug="",
        type=CUSTOM_RESOURCE_TYPE.RESOURCE.value,
        icon="fa-database",
        icon_color="",
        display_icon=False,
        role_choice="",
        role_type="",
        sort=0,
    ):
        """create_custom_resource

        Args:
            template
            name_in_schema
            title
            description=
            slug
            type
            icon=
            icon_color
            display_icon
            role_choice
            role_type=
            sort
        Returns:

        """
        if template is None:
            template = CustomResourceFixtures.create_and_save_template()

        custom_resource = CustomResource(
            template=template,
            name_in_schema=name_in_schema,
            title=title,
            description=description,
            slug=slug,
            type=type,
            icon=icon,
            icon_color=icon_color,
            display_icon=display_icon,
            role_choice=role_choice,
            role_type=role_type,
            sort=sort,
        )
        custom_resource.save()
        return custom_resource

    @staticmethod
    def get_dict_custom_resource_minus_sort():
        """get_dict_custom_resource_minus_sort

        Returns:

        """
        mock_json_path = join(dirname(realpath(__file__)), "json")
        json_path = join(mock_json_path, "custom_resource_minus_sort.json")
        with open(json_path, "r", encoding="utf-8") as json_file:
            return json.load(json_file)

    @staticmethod
    def get_dict_custom_resource_no_sort():
        """get_dict_custom_resource_no_sort

        Returns:

        """
        mock_json_path = join(dirname(realpath(__file__)), "json")
        json_path = join(mock_json_path, "custom_resource_no_sort.json")
        with open(json_path, "r", encoding="utf-8") as json_file:
            return json.load(json_file)

    @staticmethod
    def get_dict_custom_resource_no_title():
        """get_dict_custom_resource_no_title

        Returns:

        """
        mock_json_path = join(dirname(realpath(__file__)), "json")
        json_path = join(mock_json_path, "custom_resource_no_title.json")
        with open(json_path, "r", encoding="utf-8") as json_file:
            return json.load(json_file)

    @staticmethod
    def get_dict_custom_resource_no_type():
        """get_dict_custom_resource_no_type

        Returns:

        """
        mock_json_path = join(dirname(realpath(__file__)), "json")
        json_path = join(mock_json_path, "custom_resource_no_type.json")
        with open(json_path, "r", encoding="utf-8") as json_file:
            return json.load(json_file)

    @staticmethod
    def get_dict_all_custom_resource():
        """get_dict_all_custom_resource

        Returns:

        """
        mock_json_path = join(dirname(realpath(__file__)), "json")
        json_path = join(mock_json_path, "all_custom_resource.json")
        with open(json_path, "r", encoding="utf-8") as json_file:
            return json.load(json_file)

    @staticmethod
    def get_dict_custom_resource_all_resource():
        """get_dict_custom_resource_all_resource

        Returns:

        """
        mock_json_path = join(dirname(realpath(__file__)), "json")
        json_path = join(mock_json_path, "all_resource.json")
        with open(json_path, "r", encoding="utf-8") as json_file:
            return json.load(json_file)

    @staticmethod
    def get_dict_custom_resource():
        """get_dict_custom_resource

        Returns:

        """
        mock_json_path = join(dirname(realpath(__file__)), "json")
        json_path = join(mock_json_path, "custom_resource.json")
        with open(json_path, "r", encoding="utf-8") as json_file:
            return json.load(json_file)



""" REST views for the registry template version manager API
"""
from rest_framework.permissions import IsAuthenticated

from core_main_app.rest.template_version_manager.abstract_views import (
    AbstractStatusTemplateVersion,
)
from core_main_registry_app.components.version_manager import (
    api as version_manager_api,
)


class CurrentTemplateVersion(AbstractStatusTemplateVersion):
    """Update status to current"""

    permission_classes = (IsAuthenticated,)

    def status_update(self, template_object):
        """Update status to current

        Args:

            template_object: template_version

        Returns:

            TemplateVersion
        """
        return version_manager_api.set_current(
            template_object, request=self.request
        )



"""
FancyTree widget.
Based on xrmx work: https://github.com/xrmx/django-fancytree.
Modified for the registry project.
"""
from itertools import chain

from django import forms
from django.conf import settings
from django.forms.widgets import Widget
from django.utils.datastructures import MultiValueDict
from django.utils.encoding import force_str
from django.utils.html import conditional_escape
from django.utils.safestring import mark_safe
from mptt.templatetags.mptt_tags import cache_tree_children

FANCYTREE_CDN_PATH = (
    "https://cdnjs.cloudflare.com/ajax/libs/jquery.fancytree/2.38.3"
)

try:
    import simplejson as json
except ImportError:
    import json


def get_doc(node, values, count_mode):
    """Represent a node.

    Args:
        node:
        values:
        count_mode:  Add an html element to display counts next to each node (True/False).

    Returns:

    """
    if hasattr(node, "get_doc"):
        return node.get_doc(values)
    if hasattr(node, "name"):
        name = node.name
    else:
        name = str(node)

    #  Add an html element to display counts next to each node.
    if count_mode:
        count_html = "<em class='occurrences' id='{0}'></em>".format(node.pk)
        doc = {"title": "{0} {1}".format(name, count_html), "key": node.pk}
    else:
        doc = {"title": name, "key": node.pk}

    if str(node.pk) in values:
        doc["selected"] = True
        doc["expand"] = True
    return doc


def recursive_node_to_dict(node, values, count_mode):
    """recursive node to dict.

    Args:
        node:
        values:
        count_mode:

    Returns:

    """
    result = get_doc(node, values, count_mode)
    children = [
        recursive_node_to_dict(c, values, count_mode)
        for c in node.get_children()
    ]
    if children:
        expand = [c for c in children if c.get("selected", False)]
        if expand:
            result["expand"] = True
        result["folder"] = True
        result["children"] = children
    return result


def get_tree(nodes, values, count_mode):
    """get tree.

    Args:
        nodes:
        values:
        count_mode:

    Returns:

    """
    root_nodes = cache_tree_children(nodes)
    return [recursive_node_to_dict(n, values, count_mode) for n in root_nodes]


class FancyTreeWidget(Widget):
    """Fancy Tree Widget"""

    def __init__(
        self,
        attrs=None,
        choices=(),
        queryset=None,
        select_mode=3,
        count_mode=False,
    ):
        """

        Args:
            attrs:
            choices:
            queryset:
            select_mode:
            count_mode: Add an html element to display counts next to each node (True/False).

        """
        super().__init__(attrs)
        self.queryset = queryset
        self.select_mode = select_mode
        self.choices = list(choices)
        self.count_mode = count_mode

    def value_from_datadict(self, data, files, name):
        """value from datadict

        Args:
            data:
            files:
            name:

        Returns:

        """

        if isinstance(data, MultiValueDict):
            return data.getlist(name)
        return data.get(name, None)

    def render(self, name, value, attrs=None, choices=(), renderer=None):
        """render

        Args:
            name:
            value:
            attrs:
            choices:
            renderer:

        Returns:

        """
        if value is None:
            value = []
        if not isinstance(value, (list, tuple)):
            value = [value]
        has_id = attrs and "id" in attrs
        final_attrs = self.build_attrs(attrs)
        if has_id:
            output = [
                '<div id="%s" name="%s"></div>'
                % (attrs["id"], self.choices.field.label)
            ]
            id_attr = ' id="%s_checkboxes"' % (attrs["id"])
        else:
            output = ['<div name="%s"></div>' % self.choices.field.label]
            id_attr = ""
        output.append(
            '<ul style="display: none;" class="fancytree_checkboxes"%s>'
            % id_attr
        )
        str_values = set([force_str(v) for v in value])
        for i, (option_value, option_label) in enumerate(
            chain(self.choices, choices)
        ):
            if has_id:
                final_attrs = dict(
                    final_attrs, id="%s_%s" % (attrs["id"], option_value)
                )
                label_for = ' for="%s"' % final_attrs["id"]
            else:
                label_for = ""

            checkbox = forms.CheckboxInput(
                final_attrs, check_test=lambda value: value in str_values
            )
            option_value = force_str(option_value)
            rendered_cb = checkbox.render(name, option_value)
            option_label = conditional_escape(force_str(option_label))
            output.append(
                "<li><label%s>%s %s</label></li>"
                % (label_for, rendered_cb, option_label)
            )
        output.append("</ul>")
        output.append('<script type="text/javascript">')
        js_data_var = "fancytree_data_%s" % (attrs["id"].replace("-", "_"))
        if has_id:
            output.append(
                "var %s = %s;"
                % (
                    js_data_var,
                    json.dumps(
                        get_tree(self.queryset, str_values, self.count_mode)
                    ),
                )
            )
            output.append(
                """
                var defer_initFancyTree = function() {
                    $.when(
                        cachedScript( "%(fancytree)s" ),
                        $.Deferred(function( deferred ){
                            $( deferred.resolve );
                        })
                    ).done(function(){
                        $.when(
                            $.Deferred(function( deferred ){
                                $( deferred.resolve );
                            })
                        ).done(function(){
                            $("#%(id)s").fancytree({
                                extensions: ["glyph"],
                                checkbox: true,
                                icon: false,
                                selectMode: %(select_mode)d,
                                source: %(js_var)s,
                                debugLevel: %(debug)d,
                                glyph: {
                                    map: {
                                        expanderClosed: "fa-solid fa-caret-right",
                                        expanderLazy: "fa-solid fa-caret-right",
                                        expanderOpen: "fa-solid fa-caret-down",
                                        checkbox: "fa-regular fa-square",
                                        checkboxSelected: "fa-regular fa-square-check",
                                        checkboxUnknown: "fa-regular fa-square-minus",
                                    }
                                },
                                customTag : {
                                    tag: "div"
                                },
                                _classNames: {
                                    active: "no-css",
                                    focused: "no-css"
                                },
                                select: function(event, data) {
                                    $('#%(id)s_checkboxes').find('input[type=checkbox]').prop('checked', false);
                                    var selNodes = data.tree.getSelectedNodes();
                                    var selKeys = $.map(selNodes, function(node){
                                           $('#%(id)s_' + (node.key)).prop('checked', true);
                                           return node.key;
                                    });
                                    // trigger the event fancy_tree_select
                                    $(document).trigger("fancy_tree_select_event", data);
                                },
                                click: function(event, data) {
                                    var node = data.node;
                                    if (event.targetType == "fancytreeclick")
                                        node.toggleSelected();
                                },
                                keydown: function(event, data) {
                                    var node = data.node;
                                    if (event.which == 32) {
                                        node.toggleSelected();
                                        return false;
                                    }
                                },
                                init: function(event, data) {
                                    // Render all nodes even if collapsed
                                    data.tree.getRootNode().render(force=true, deep=true);
                                    // set a timeout to let the tree finish its rendering
                                    setTimeout(function(){
                                        // trigger the event fancy_tree_ready
                                        $(document).trigger("fancy_tree_ready_event", data);
                                    }, 200);
                                },
                            });
                        });
                    });
                };
                onjQueryReady(defer_initFancyTree);

                """
                % {
                    "id": attrs["id"],
                    "js_var": js_data_var,
                    "debug": settings.DEBUG and 1 or 0,
                    "select_mode": self.select_mode,
                    "fancytree": f"{FANCYTREE_CDN_PATH}/jquery.fancytree-all-deps.min.js",
                }
            )
        output.append("</script>")
        return mark_safe("\n".join(output))

    class Media:
        """Media"""

        js = ("core_explore_common_app/common/js/tools.js",)

        css = {
            "all": (
                f"{FANCYTREE_CDN_PATH}/skin-bootstrap/ui.fancytree.min.css",
                "core_main_registry_app/user/css/fancytree/fancytree.custom.css",
            )
        }



"""
Version Manager API
"""

from core_main_app.commons import exceptions as exceptions
from core_main_app.components.version_manager import api as version_manager_api
from core_main_registry_app.components.custom_resource import (
    api as custom_resource_api,
)


# NOTE: access control by main set current
def set_current(version, request):
    """Set the current version of the object, then saves it.

    Args:
        version:
        request:

    Returns:

    """
    if custom_resource_api.get_all_by_template(version).count() == 0:
        raise exceptions.ApiError(
            "Please link a Custom Resources file to this template before setting it to current."
        )
    version_manager_api.set_current(version, request=request)



"""
Refinements creation.
"""
from core_main_registry_app.constants import UNSPECIFIED_LABEL
from core_main_registry_app.utils.refinement.tools import xsd_refinements


def init_refinements(template):
    """Init the refinements for the given template. Categories used as refinement (search page...).

    Args:
        template:

    Returns:

    """
    from core_main_registry_app.components.refinement import (
        api as refinement_api,
    )

    try:
        # Check if refinements already exist.
        if not refinement_api.check_refinements_already_exist_by_template_hash(
            template.hash
        ):
            refinements_trees = xsd_refinements.loads_refinements_trees(
                template
            )
            # Create refinements.
            for root, tree in list(refinements_trees.items()):
                refinement = refinement_api.create_and_save(
                    name=root.title,
                    xsd_name=root.xsd_name,
                    template_hash=template.hash,
                )
                # Create categories.
                create_categories(tree, refinement)
    except Exception as exception:
        raise Exception(
            f"Impossible to init the refinements. An error occurred while retrieving "
            f"the template: {str(exception)}"
        )


def create_categories(tree, refinement):
    """Create the refinement categories.

    Args:
        tree: Tree of categories.
        refinement: Refinement.

    """
    # Iterate over the categories.
    for key, leaves in list(tree.items()):
        _create_sub_categories(key, leaves, refinement)


def _create_sub_categories(key, leaves, refinement, parent=None):
    """Create all sub categories.

    Args:
        key:
        leaves:
        refinement:
        parent:

    """
    from core_main_registry_app.components.category import api as category_api

    # No children. Only create the category.
    if len(leaves) == 0:
        category_api.create_and_save(
            name=key.title,
            path=key.path,
            value=key.value,
            parent=parent,
            refinement=refinement,
        )
    elif len(leaves) > 0:

        value = (
            key.value
            if UNSPECIFIED_LABEL in key.title
            else key.value_as_category()
        )

        # Create the category and become a parent.
        parent = category_api.create_and_save(
            name=key.title,
            path=key.path,
            value=value,
            parent=parent,
            refinement=refinement,
        )
        # For each child.
        for key, value in sorted(leaves.items()):
            # Create sub categories.
            _create_sub_categories(key, value, refinement, parent=parent)



"""
Management of the refinements' Tree from a schema.
"""

import logging
from collections import OrderedDict

from core_main_app.utils.xsd_flattener.xsd_flattener_database_url import (
    XSDFlattenerDatabaseOrURL,
)
from core_parser_app.tools.parser.utils.xml import get_app_info_options
from xml_utils.commons.constants import LXML_SCHEMA_NAMESPACE
from xml_utils.xsd_tree.operations.namespaces import (
    get_namespaces,
    get_target_namespace,
)
from xml_utils.xsd_tree.xsd_tree import XSDTree
from core_main_registry_app.utils.refinement.tools import tree

logger = logging.getLogger(
    "core_main_registry_app.utils.refinement.tools.xsd_refinements"
)


def loads_refinements_trees(template):
    """Load refinements for the given template.

    Args:
        template:

    Returns:

    """
    # Get the flatten schema
    ref_xml_schema_content = _get_flatten_schema(template)
    xml_doc_tree = XSDTree.build_tree(ref_xml_schema_content)
    # Get the target namespace
    target_ns_prefix = _get_target_namespace_prefix(
        ref_xml_schema_content, xml_doc_tree
    )
    target_ns_prefix = (
        "{}:".format(target_ns_prefix) if target_ns_prefix != "" else ""
    )

    # Iterate over the simple types.
    simple_types = xml_doc_tree.findall(
        "./{0}simpleType".format(LXML_SCHEMA_NAMESPACE)
    )
    trees = OrderedDict()
    for simple_type in simple_types:
        try:
            # Get all the enumerations
            enums = simple_type.findall(
                "./{0}restriction/{0}enumeration".format(LXML_SCHEMA_NAMESPACE)
            )
            if len(enums) > 0:
                # Get the corresponding element
                element = xml_doc_tree.findall(
                    ".//{0}element[@type='{1}']".format(
                        LXML_SCHEMA_NAMESPACE,
                        target_ns_prefix + simple_type.attrib["name"],
                    )
                )
                if len(element) > 1:
                    logger.error(
                        "More than one element using the enumeration ({0})".format(
                            str(len(element))
                        )
                    )
                else:
                    element = element[0]
                    # get the label of refinements
                    element_name, element_label = _get_element_info(
                        element, xml_doc_tree, target_ns_prefix
                    )
                    query = []

                    # Build the path to access the element (dot notation)
                    while element is not None:
                        if element.tag == "{0}element".format(
                            LXML_SCHEMA_NAMESPACE
                        ):
                            query.insert(0, element.attrib["name"])
                        elif element.tag == "{0}simpleType".format(
                            LXML_SCHEMA_NAMESPACE
                        ):
                            element = _get_simple_type_or_complex_type_info(
                                xml_doc_tree, target_ns_prefix, element, query
                            )
                        elif element.tag == "{0}complexType".format(
                            LXML_SCHEMA_NAMESPACE
                        ):
                            element = _get_simple_type_or_complex_type_info(
                                xml_doc_tree, target_ns_prefix, element, query
                            )
                        elif element.tag == "{0}extension".format(
                            LXML_SCHEMA_NAMESPACE
                        ):
                            element = _get_extension_info(
                                xml_doc_tree, element, query
                            )

                        element = element.getparent()

                    dot_query = ".".join(query)
                    # Build the corresponding refinement tree
                    trees = tree.build_tree(
                        tree=trees,
                        element_name=element_name,
                        element_display_name=element_label,
                        enums=enums,
                        dot_query=dot_query,
                    )
        except Exception as exception:
            # Log the exception
            logger.warning(str(exception))

    return trees


def _get_flatten_schema(template):
    """Get the flatten schema of the given template.

    Args:
        template:

    Returns:

    """
    return XSDFlattenerDatabaseOrURL(template.content, request=None).get_flat()


def _get_target_namespace_prefix(ref_xml_schema_content, xml_doc_tree):
    """Get the target namespace prefix.

    Args:
        ref_xml_schema_content:
        xml_doc_tree:

    Returns:

    """
    namespaces = get_namespaces(ref_xml_schema_content)
    target_namespace, target_ns_prefix = get_target_namespace(
        xml_doc_tree, namespaces
    )

    return target_ns_prefix


def _get_element_info(element, xml_doc_tree, target_ns_prefix):
    """Get the element label.

    Args:
        element:
        xml_doc_tree:
        target_ns_prefix:

    Returns:
        Label: string.

    """
    # By default, use the element's app_info
    app_info = get_app_info_options(element)
    # Check if the element is embedded in a choice.
    # If yes, we use the first parent complexType to get the label.
    parent = element.getparent()
    if parent is not None and parent.tag == "{0}choice".format(
        LXML_SCHEMA_NAMESPACE
    ):
        while parent.getparent() is not None:
            parent = parent.getparent()
            if parent.tag == "{0}complexType".format(LXML_SCHEMA_NAMESPACE):
                parent = _get_simple_type_or_complex_type_info(
                    xml_doc_tree, target_ns_prefix, parent
                )
                app_info = get_app_info_options(parent)
                break

    # Get the label
    name = parent.attrib["name"] if "name" in parent.attrib else ""
    label = app_info["label"] if "label" in app_info else name
    label = label if label is not None else name

    return name, label


def _get_simple_type_or_complex_type_info(
    xml_doc_tree, target_ns_prefix, element, query=None
):
    """Get simple type / complex type information.

    Args:
        xml_doc_tree:
        target_ns_prefix:
        element:
        query:

    Returns:

    """
    try:
        to_search_element = xml_doc_tree.findall(
            ".//{0}element[@type='{1}']".format(
                LXML_SCHEMA_NAMESPACE,
                target_ns_prefix + element.attrib["name"],
            )
        )
        if len(to_search_element) == 0:
            logger.debug(
                "No element using the enumeration ({0})".format(
                    str(len(element))
                )
            )
            element = _find_extension(xml_doc_tree, target_ns_prefix, element)
        elif len(to_search_element) > 1:
            logger.error(
                "More than one element using the enumeration ({0})".format(
                    str(len(element))
                )
            )
        else:
            element = to_search_element[0]
            if query is not None:
                query.insert(0, element.attrib["name"])
    except Exception as exception:
        raise Exception(
            "Impossible to get simple type / complex type information: {0}".format(
                str(exception)
            )
        )

    return element


def _get_extension_info(xml_doc_tree, element, query=None):
    """Get extension information.

    Args:
        xml_doc_tree:
        element:
        query:

    Returns:

    """
    try:
        to_search_element = xml_doc_tree.findall(
            ".//{0}element[@type='{1}']".format(
                LXML_SCHEMA_NAMESPACE, element.attrib["base"]
            )
        )
        if len(to_search_element) == 0:
            logger.debug(
                "No element using the enumeration ({0})".format(
                    str(len(element))
                )
            )
        elif len(to_search_element) > 1:
            logger.error(
                "More than one element using the enumeration ({0})".format(
                    str(len(element))
                )
            )
        else:
            element = to_search_element[0]
            if query is not None:
                query.insert(0, element.attrib["name"])
    except Exception as exception:
        raise Exception(
            "Impossible to get the extension information: {0}".format(
                str(exception)
            )
        )

    return element


def _find_extension(xml_doc_tree, target_ns_prefix, element):
    """Find the element extension.

    Args:
        xml_doc_tree:
        target_ns_prefix:
        element:

    Returns:

    """
    try:
        to_search_element = xml_doc_tree.findall(
            ".//{0}extension[@base='{1}']".format(
                LXML_SCHEMA_NAMESPACE,
                target_ns_prefix + element.attrib["name"],
            )
        )
        if len(to_search_element) == 0:
            logger.debug(
                "The enumeration ({0}) was not found".format(str(len(element)))
            )
        elif len(to_search_element) > 1:
            logger.error(
                "More than one enumeration using the element ({0})".format(
                    str(len(element))
                )
            )
        else:
            element = to_search_element[0]
    except Exception as exception:
        raise Exception(
            "Impossible to get the extension information: {0}".format(
                str(exception)
            )
        )

    return element



""" Unit Test for Category API
"""

from django.test import TestCase

from core_main_app.commons import exceptions as exceptions
from core_main_registry_app.components.category import (
    api as category_api,
)
from core_main_registry_app.components.category.models import (
    Category,
)
from core_main_registry_app.components.refinement.models import (
    Refinement,
)


class TestCreateAndSaveCategory(TestCase):
    """
    Test Create And Save Category
    """

    def test_create_and_save_category_creates_category(self):
        """test_create_and_save_category_creates_category"""

        # Arrange
        refinement = Refinement.create_and_save("Refinement", "", "")
        # Act
        result = category_api.create_and_save(
            "Category", "/Path", "", None, refinement
        )
        # Assert
        self.assertIsInstance(result, Category)
        self.assertEqual(result.name, "Category")


class TestCategoryGetAllFilteredByRefinementId(TestCase):
    """
    Test Category Get All Filtered By Refinement Id
    """

    def test_category_get_all_filtered_by_refinement_id_returns_list_of_categories(
        self,
    ):
        """test_category_get_all_filtered_by_refinement_id_returns_list_of_categories"""
        # Act
        result = category_api.get_all_filtered_by_refinement_id(1)
        # Assert
        for obj in list(result):
            self.assertTrue(isinstance(obj, Category))

    def test_category_get_all_filtered_by_refinement_id_returns_list(self):
        """test_category_get_all_filtered_by_refinement_id_returns_list"""
        # Arrange
        category = create_category()
        # Act
        result = category_api.get_all_filtered_by_refinement_id(
            category.refinement.id
        )
        # Assert
        self.assertEqual(len(result), 1)
        self.assertTrue(category in result)


class TestCategoryGetById(TestCase):
    """
    Test Category Get By Id
    """

    def test_category_get_by_id_raises_does_not_exist_exception(self):
        """test_category_get_by_id_raises_does_not_exist_exception"""
        # Act # Assert
        with self.assertRaises(exceptions.DoesNotExist):
            category_api.get_by_id(-1)

    def test_category_get_all_filtered_by_refinement_id_returns_list(self):
        """test_get_all_filtered_by_refinement_id_returns_list"""
        # Arrange
        category = create_category()
        # Act
        result = category_api.get_by_id(category.id)
        # Assert
        self.assertIsInstance(result, Category)


class TestCategoryGetAll(TestCase):
    """
    Test Category Get All
    """

    def test_category_get_all_returns_list_of_categories(self):
        """test_category_get_all_returns_list_of_categories"""
        # Act
        result = category_api.get_all()
        # Assert
        for obj in list(result):
            self.assertTrue(isinstance(obj, Category))

    def test_category_get_all_returns_list(self):
        """test_category_get_all_returns_list"""
        # Arrange
        category = create_category()
        # Act
        result = category_api.get_all()
        # Assert
        self.assertTrue(category in result)


class TestCategoryGetAllCategoriesIdsFromNameAndRefinementId(TestCase):
    """
    Test Category Get All Categories Ids From Name And Refinement Id
    """

    def test_category_get_all_categories_ids_from_name_and_refinement_id_raises_does_not_exist_exception(
        self,
    ):
        """test_category_get_all_categories_ids_from_name_and_refinement_id_raises_does_not_exist_exception"""
        # Act # Assert
        with self.assertRaises(exceptions.DoesNotExist):
            category_api.get_all_categories_ids_from_name_and_refinement_id(
                "Category", 1
            )

    def test_category_get_all_categories_ids_from_name_and_refinement_id_returns_list(
        self,
    ):
        """test_category_get_all_categories_ids_from_name_and_refinement_id_returns_list"""
        # Arrange
        category = create_category()
        # Act
        result = (
            category_api.get_all_categories_ids_from_name_and_refinement_id(
                "Category", category.refinement.id
            )
        )
        # Assert
        self.assertEqual(len(result), 1)


class TestCategoryGetAllCategoriesIdsByParentSlugAndRefinementId(TestCase):
    """
    Test Category Get All Categories Ids By Parent Slug And RefinementId
    """

    def test_get_all_categories_ids_by_parent_slug_and_refinement_id_raises_does_not_exist_exception(
        self,
    ):
        """test_get_all_categories_ids_by_parent_slug_and_refinement_id_raises_does_not_exist_exception"""
        # Act # Assert
        with self.assertRaises(exceptions.DoesNotExist):
            category_api.get_all_categories_ids_by_parent_slug_and_refinement_id(
                "parent", 1
            )

    def test_get_all_categories_ids_by_parent_slug_and_refinement_id_returns_list(
        self,
    ):
        """test_get_all_categories_ids_by_parent_slug_and_refinement_id_returns_list"""
        # Arrange
        category = create_category()
        # Act
        result = category_api.get_all_categories_ids_by_parent_slug_and_refinement_id(
            "", category.refinement.id
        )
        # Assert
        self.assertEqual(len(result), 1)


def create_category():
    """create_refinement

    Return:
    """
    refinement = Refinement.create_and_save("Refinement", "", "")
    return category_api.create_and_save(
        "Category", "/Path", "", None, refinement
    )



""" REST views for the workspace API
"""
from rest_framework.permissions import IsAuthenticated

from core_main_app.rest.workspace.views import WorkspaceList
from core_main_registry_app.rest.workspace.serializers import (
    WorkspaceSerializer,
)


class WorkspaceListRegistry(WorkspaceList):
    """List all user Workspace, or create a new one"""

    permission_classes = (IsAuthenticated,)
    serializer = WorkspaceSerializer



""" Handle refinement signals
"""
from logging import getLogger

from billiard.exceptions import SoftTimeLimitExceeded
from django.db.models.signals import post_save

from core_main_app.components.template.models import Template
from core_main_registry_app.tasks import init_refinement_task

logger = getLogger(__name__)


def init():
    """Connect to template object events."""
    post_save.connect(post_save_template, sender=Template)


def post_save_template(sender, instance, **kwargs):
    """Method executed after saving of a Template object.
    Args:
        sender:
        instance: template object.
        **kwargs:
    """
    try:
        # start asynchronous task
        if kwargs["created"]:
            init_refinement_task.delay(str(instance.id))
    except (TimeoutError, SoftTimeLimitExceeded) as ex:
        logger.error("Timeout while generating refinements: %s ", str(ex))
    except Exception as ex:
        logger.error(
            "Error happened while generating refinements:  %s ", str(ex)
        )



""" Integration Test for Custom Resource API
"""
from django.core.exceptions import ValidationError
from django.utils.text import slugify
from unittest.mock import patch

from core_main_app.commons import exceptions as exceptions
from core_main_app.utils.integration_tests.integration_base_test_case import (
    IntegrationBaseTestCase,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import create_mock_request
from core_main_registry_app.components.custom_resource import (
    api as custom_resource_api,
)
from core_main_registry_app.components.custom_resource.models import (
    CustomResource,
)
from core_main_registry_app.constants import CUSTOM_RESOURCE_TYPE
from tests.components.custom_resource.fixtures.fixtures import (
    CustomResourceFixtures,
)

fixtureCustomResource = CustomResourceFixtures()


class TestCreateAndSaveCustomResource(IntegrationBaseTestCase):
    """
    Test Create And Save Custom Resource
    """

    fixture = fixtureCustomResource

    def test_create_all_custom_resource_return_collection_of_custom_resource(
        self,
    ):
        """test_create_all_custom_resource_return_collection_of_custom_resource

        Returns:
        """

        # Act
        template = self.fixture.create_and_save_template()
        custom_resource_api.parse_and_save(
            self.fixture.get_dict_all_custom_resource(), template
        )
        result = custom_resource_api.get_all_by_template(template)
        # Assert
        self.assertTrue(
            all(isinstance(item, CustomResource) for item in result)
        )

    def test_create_custom_resource_return_slug_is_not_none(self):
        """test_create_custom_resource_return_slug_is_not_none

        Returns:
        """

        # Act
        template = self.fixture.create_and_save_template()
        custom_resource_api.parse_and_save(
            self.fixture.get_dict_custom_resource(), template
        )
        result = custom_resource_api.get_all_by_template(template)
        # Assert
        self.assertIsNotNone(result[0].slug)

    def test_create_custom_resource_type_all_return_refinements_is_empty(self):
        """test_create_custom_resource_type_all_return_refinements_is_empty

        Returns:
        """
        # Act
        template = self.fixture.create_and_save_template()
        custom_resource_api.parse_and_save(
            self.fixture.get_dict_custom_resource_all_resource(), template
        )
        result = custom_resource_api.get_all_by_template(template)
        # Assert
        self.assertEqual(len(result[0].refinements), 0)

    def test_create_custom_resource_return_refinements_is_not_none(self):
        """test_create_custom_resource_return_refinements_is_not_none

        Returns:
        """
        # Act
        template = self.fixture.create_and_save_template()
        custom_resource_api.parse_and_save(
            self.fixture.get_dict_custom_resource(), template
        )
        result = custom_resource_api.get_all_by_template(template)
        # Assert
        self.assertIsNotNone(result[0].refinements)

    def test_create_custom_resource_return_refinements_length(self):
        """test_create_custom_resource_return_refinements_length

        Returns:
        """
        # Act
        template = self.fixture.create_and_save_template()
        custom_resource_api.parse_and_save(
            self.fixture.get_dict_custom_resource(), template
        )
        result = custom_resource_api.get_all_by_template(template)
        # Assert
        self.assertEqual(len(result[0].refinements), 1)

    def test_create_custom_resource_return_refinements_value(self):
        """test_create_custom_resource_return_refinements_value

        Returns:
        """
        # Act
        template = self.fixture.create_and_save_template()
        custom_resource_api.parse_and_save(
            self.fixture.get_dict_custom_resource(), template
        )
        result = custom_resource_api.get_all_by_template(template)
        # Assert
        self.assertEqual(result[0].refinements[0], "unspecified Organization")

    def test_create_custom_resource_return_slug(self):
        """test_create_custom_resource_return_slug

        Returns:
        """
        # Act
        template = self.fixture.create_and_save_template()
        custom_resource_api.parse_and_save(
            self.fixture.get_dict_custom_resource(), template
        )
        result = custom_resource_api.get_all_by_template(template)
        # Assert
        self.assertEqual(result[0].slug, slugify(result[0].title))

    def test_create_custom_resource_return_type(self):
        """test_create_custom_resource_return_type

        Returns:
        """
        # Act
        template = self.fixture.create_and_save_template()
        custom_resource_api.parse_and_save(
            self.fixture.get_dict_custom_resource(), template
        )
        result = custom_resource_api.get_all_by_template(template)
        # Assert
        self.assertEqual(result[0].type, CUSTOM_RESOURCE_TYPE.RESOURCE.value)

    def test_create_all_resource_custom_resource_return_type_is_not_none(self):
        """test_create_all_resource_custom_resource_return_type_is_not_none

        Returns:
        """
        # Act
        template = self.fixture.create_and_save_template()
        custom_resource_api.parse_and_save(
            self.fixture.get_dict_all_custom_resource(), template
        )
        result = custom_resource_api.get_all_by_template(template)
        # Assert
        self.assertIsNotNone(all(item.type for item in result))

    def test_create_custom_resource_minus_sort_return_except(self):
        """test_create_custom_resource_minus_sort_return_except

        Returns:
        """
        # Act
        template = self.fixture.create_and_save_template()
        # Assert
        with self.assertRaises(exceptions.ModelError):
            custom_resource_api.parse_and_save(
                self.fixture.get_dict_custom_resource_minus_sort(), template
            )

    def test_create_custom_resource_no_title_return_except(self):
        """test_create_custom_resource_no_title_return_except

        Returns:
        """
        # Act
        template = self.fixture.create_and_save_template()
        # Assert
        with self.assertRaises(exceptions.ModelError):
            custom_resource_api.parse_and_save(
                self.fixture.get_dict_custom_resource_no_title(), template
            )

    def test_create_custom_resource_no_sort_return_except(self):
        """test_create_custom_resource_no_sort_return_except

        Returns:
        """
        # Act
        template = self.fixture.create_and_save_template()
        # Assert
        with self.assertRaises(exceptions.ModelError):
            custom_resource_api.parse_and_save(
                self.fixture.get_dict_custom_resource_no_sort(), template
            )

    def test_create_custom_resource_wrong_type_return_except(self):
        """test_create_custom_resource_wrong_type_return_except

        Returns:
        """
        # Act
        template = self.fixture.create_and_save_template()
        # Assert
        with self.assertRaises(ValidationError):
            custom_resource = CustomResource(
                template=template,
                title="title",
                type="wrong",
                icon="icon",
                sort=0,
            )
            custom_resource.save()


class TestGetAllByTemplate(IntegrationBaseTestCase):
    """
    Test Get All By Template
    """

    fixture = fixtureCustomResource

    def test_get_all_by_template_return_all(self):
        """test_get_all_by_template_return_all

        Returns:
        """

        # Act
        length = len(self.fixture.get_dict_all_custom_resource())
        template = self.fixture.create_and_save_template()
        custom_resource_api.parse_and_save(
            self.fixture.get_dict_all_custom_resource(), template
        )
        result = custom_resource_api.get_all_by_template(template)
        # Assert
        self.assertEqual(length, len(result))


class TestGetByRoleForCurrentTemplate(IntegrationBaseTestCase):
    """
    Test Get By Role For Current Template
    """

    fixture = fixtureCustomResource

    @patch(
        "core_main_registry_app.components.custom_resource.api._get_current_template"
    )
    def test_get_by_role_for_current_template_returns_custom_resource(
        self, _get_current_template
    ):
        """test_get_by_role_for_current_template_returns_custom_resource

        Returns:
        """

        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(user=mock_user)
        _get_current_template.return_value = self.fixture.template
        # Act
        custom_resource = custom_resource_api.get_by_role_for_current_template(
            self.fixture.custom_resource.role_choice, request=mock_request
        )
        # Assert
        self.assertTrue(isinstance(custom_resource, CustomResource))


class TestGetByCurrentTemplateAndSlug(IntegrationBaseTestCase):
    """
    Test Get By Current Template And Slug
    """

    fixture = fixtureCustomResource

    @patch(
        "core_main_registry_app.components.custom_resource.api._get_current_template"
    )
    def test_get_by_current_template_and_slug_returns_custom_resource(
        self, _get_current_template
    ):
        """test_get_by_current_template_and_slug_returns_custom_resource

        Returns:
        """

        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(user=mock_user)
        _get_current_template.return_value = self.fixture.template
        # Act
        custom_resource = custom_resource_api.get_by_current_template_and_slug(
            self.fixture.custom_resource.slug, request=mock_request
        )
        # Assert
        self.assertTrue(isinstance(custom_resource, CustomResource))

    @patch(
        "core_main_registry_app.components.custom_resource.api._get_current_template"
    )
    def test_get_by_current_template_and__incorrect_slug_raises_error(
        self, _get_current_template
    ):
        """test_get_by_current_template_and__incorrect_slug_raises_error

        Returns:
        """

        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(user=mock_user)
        _get_current_template.return_value = self.fixture.template
        # Act
        with self.assertRaises(exceptions.DoesNotExist):
            custom_resource_api.get_by_current_template_and_slug(
                "incorrect slug", mock_request
            )


class TestDeleteCustomResourcesByTemplate(IntegrationBaseTestCase):
    """
    Test Delete Custom Resources By Template
    """

    fixture = fixtureCustomResource

    def test_delete_custom_resources_by_template_return_0(self):
        """test_delete_custom_resources_by_template_return_0

        Returns:
        """

        # Act
        self.fixture.insert_data()
        custom_resource_api.delete_custom_resources_by_template(
            self.fixture.template
        )
        result = custom_resource_api.get_all_by_template(self.fixture.template)
        # Assert
        self.assertEqual(0, len(result))


class TestSaveList(IntegrationBaseTestCase):
    """
    Test Save List
    """

    fixture = fixtureCustomResource

    def test_save_list(self):
        """test_save_list

        Returns:
        """

        # Act
        template = self.fixture.create_and_save_template()
        custom_resource1 = CustomResource(
            template=template,
            title="title1",
            type="resource",
            icon="icon",
            sort=0,
        )
        custom_resource2 = CustomResource(
            template=template,
            title="title2",
            type="resource",
            icon="icon",
            sort=1,
        )
        list_custom = [custom_resource1, custom_resource2]
        custom_resource_api.save_list(list_custom)
        result = custom_resource_api.get_all_by_template(template)
        # Assert
        self.assertEqual(len(list_custom), len(result))


class TestReplaceCustomResourceByTemplate(IntegrationBaseTestCase):
    """
    Test Replace Custom Resource By Template
    """

    fixture = fixtureCustomResource

    def test_replace_custom_resources_by_template_return_ok(self):
        """test_replace_custom_resources_by_template_return_ok"""

        # Check
        template = self.fixture.create_and_save_template()
        result = custom_resource_api.get_all_by_template(template)
        self.assertEqual(0, len(result))
        # Act
        custom_resource_api.replace_custom_resources_by_template(
            template, self.fixture.get_dict_all_custom_resource()
        )
        result = custom_resource_api.get_all_by_template(template)
        # Assert
        self.assertEqual(10, len(result))

    def test_replace_custom_resources_by_template_check_old(self):
        """test_replace_custom_resources_by_template_check_old

        Returns:
        """

        # Check
        template = self.fixture.create_and_save_template()
        custom_resource_api.parse_and_save(
            self.fixture.get_dict_custom_resource_all_resource(), template
        )
        result = custom_resource_api.get_all_by_template(template)
        old_cr = list(result)[0]
        self.assertEqual(1, len(result))
        # Act
        custom_resource_api.replace_custom_resources_by_template(
            template, self.fixture.get_dict_all_custom_resource()
        )
        result = custom_resource_api.get_all_by_template(template)
        # Assert
        self.assertFalse(old_cr in result)

    def test_replace_custom_resources_by_template_return_raise(self):
        """test_replace_custom_resources_by_template_return_raise

        Returns:
        """

        # Act
        template = self.fixture.create_and_save_template()
        # Assert
        with self.assertRaises(exceptions.ModelError):
            custom_resource_api.replace_custom_resources_by_template(
                template, self.fixture.get_dict_custom_resource_no_title()
            )



""" Category API
"""

from core_main_registry_app.components.category.models import Category


def create_and_save(name, path, value, parent, refinement):
    """Create and save a category.

    Args:
        name:
        path:
        value:
        parent:
        refinement:

    Returns:

    """

    # Save category
    return Category.create_and_save(
        name=name, path=path, value=value, parent=parent, refinement=refinement
    )


def get_all_filtered_by_refinement_id(refinement_id):
    """Get all categories by refinement id.

    Parameters:
            refinement_id:

    Returns: data collection

    """
    return Category.get_all_filtered_by_refinement_id(refinement_id)


def get_by_id(category_id):
    """Get category by its id.

    Parameters:
        category_id:

    Returns:
        Category object

    """
    return Category.get_by_id(category_id)


def get_all_categories_ids_by_parent_slug_and_refinement_id(
    parent_slug, refinement_id
):
    """Get a list of all category ids by parent_slug and refinement id.

    Args:
        parent_slug:
        refinement_id:

    Returns:

    """
    # Get categories
    categories = Category.get_all_categories_by_parent_slug_and_refinement_id(
        parent_slug, refinement_id
    )
    # Return a list of category ids
    return list(categories.values_list("id", flat=True))


def get_all():
    """Get all categories

    Returns:

    """
    return Category.get_all()


def get_all_categories_ids_from_name_and_refinement_id(name, refinement_id):
    """Get a list of all category ids by name and refinement id.

    Args:
        name:
        refinement_id:

    Returns:

    """
    return list(
        Category.get_all_categories_ids_from_name_and_refinement_id(
            name, refinement_id
        ).values_list("id", flat=True)
    )



""" Data's registry api
"""
import random
import string

import core_main_app.components.data.api as data_api
from core_main_app.access_control.decorators import access_control
from core_main_app.commons import exceptions as exceptions
from core_main_app.components.workspace import api as workspace_api
from core_main_app.utils.datetime import datetime_now
from core_main_app.utils.labels import get_data_label
from core_main_registry_app.commons.constants import DataStatus
from core_main_registry_app.components.data.access_control import (
    can_publish_data,
)
from core_main_registry_app.system.api import is_local_id_already_used
from core_main_registry_app.utils.role.extraction import role_extraction
from xml_utils.xsd_tree.xsd_tree import XSDTree


def get_role(data):
    """Get the list of role saved in the data's dict content

    Args:
        data:

    Returns:

    """
    return role_extraction(data.get_dict_content())


@access_control(can_publish_data)
def publish(data, user):
    """Assign data to a workspace.

    Args:
        data:
        user:

    Returns:

    """
    # Check if  data has already been published
    if data.workspace is not None and data.workspace.is_public:
        raise exceptions.ApiError(get_data_label() + " already published")
    data.workspace = workspace_api.get_global_workspace()
    data.last_modification_date = datetime_now()
    return data.save()


def set_status(data, status, request):
    """Set the status of a data

    Args:
        data:
        status:
        request:

    Returns: Data

    """
    if status == DataStatus.DELETED and (
        data.workspace is None or data.workspace.is_public is False
    ):
        raise exceptions.ModelError(
            "the "
            + get_data_label()
            + " should be published if the targeted status is 'Deleted'"
        )

    # build the xsd tree
    xml_tree = XSDTree.build_tree(data.xml_content)
    # get the root
    root = xml_tree.getroot()
    # and change the attribute
    root.attrib["status"] = status
    # update the xml content
    data.xml_content = XSDTree.tostring(xml_tree)
    # upsert the data
    return data_api.upsert(data, request=request)


def get_status(data):
    """Get the status saved in the data's dict content

    Args:
        data:

    Returns:

    """
    try:
        return data.get_dict_content()["Resource"]["@status"]
    except KeyError:
        return None


def generate_unique_local_id(length_id):
    """Generate a unique ID with the given length

    Args:
        length_id:

    Returns:

    """
    # we generate a local id
    local_id = "".join(
        random.choice(string.ascii_uppercase + string.digits)
        for _ in range(length_id)
    )
    # we make sure this local id does not exist in db
    while is_local_id_already_used(local_id).count() > 0:
        # otherwise we generate one until then
        local_id = "".join(
            random.choice(string.ascii_uppercase + string.digits)
            for _ in range(length_id)
        )
    return local_id



""" Unit tests for rest apis
"""
from unittest.case import TestCase
from core_main_registry_app.rest import urls as rest_urls


class TestRestUrls(TestCase):
    def test_template_urls_in_urls(self):
        """test_template_urls_in_urls"""
        self.assertTrue(
            "core_main_app_rest_global_template_list"
            in str(rest_urls.urlpatterns)
        )
        self.assertTrue(
            "core_main_app_rest_user_template_list"
            in str(rest_urls.urlpatterns)
        )



""" Set of functions to define the rules for access control in registry
"""
from core_main_app.access_control.api import check_can_write
from core_main_app.components.data.access_control import has_perm_publish_data


def can_publish_data(func, data, user):
    """Can user publish data.

    Args:
        func:
        data:
        user:

    Returns:

    """
    if user.is_superuser:
        return func(data, user)

    has_perm_publish_data(user)
    check_can_write(data, user)
    return func(data, user)



""""Discover rules for core main registry app
"""
import json
import logging
from os.path import join

from django.contrib.staticfiles import finders
from django.db.migrations.exceptions import BadMigrationError

from core_main_app.components.template_version_manager.models import (
    TemplateVersionManager,
)
from core_main_app.components.template_xsl_rendering import (
    api as template_xsl_rendering_api,
)
from core_main_app.components.xsl_transformation import (
    api as xslt_transformation_api,
)
from core_main_app.components.xsl_transformation.models import (
    XslTransformation,
)
from core_main_app.utils.file import read_file_content
from core_main_registry_app import settings as main_registry_settings
from core_main_registry_app.components.custom_resource import (
    api as custom_resource_api,
)
from core_main_registry_app.components.custom_resource.models import (
    CustomResource,
)
from core_main_registry_app.components.refinement.models import Refinement
from core_main_registry_app.system import api as registry_system_api
from core_main_registry_app.utils.refinement import refinement

logger = logging.getLogger(__name__)


def init_registry():
    """Initialize the registry.
       Add the registry template, refinements, and resources.

    Returns:
    """
    # Check if data were previously inserted
    if (
        TemplateVersionManager.objects.count() > 0
        or CustomResource.objects.count() > 0
        or Refinement.objects.count() > 0
        or XslTransformation.objects.filter(
            name=main_registry_settings.LIST_XSL_FILENAME
        ).count()
        > 0
        or XslTransformation.objects.filter(
            name=main_registry_settings.DETAIL_XSL_FILENAME
        ).count()
        > 0
    ):
        logger.warning("The database is not empty. Skipping migration.")
        return

    # Check if file settings are set
    if (
        not main_registry_settings.REGISTRY_XSD_FILENAME
        or not main_registry_settings.REGISTRY_XSD_FILEPATH
        or not main_registry_settings.CUSTOM_REGISTRY_FILE_PATH
        or not main_registry_settings.XSL_FOLDER_PATH
        or not main_registry_settings.LIST_XSL_FILENAME
        or not main_registry_settings.DETAIL_XSL_FILENAME
    ):
        raise BadMigrationError(
            "Please configure the REGISTRY_XSD_FILENAME, "
            "REGISTRY_XSD_FILEPATH, CUSTOM_REGISTRY_FILE_PATH, "
            "XSL_FOLDER_PATH, LIST_XSL_FILENAME and DETAIL_XSL_FILENAME "
            "settings in your project."
        )

    # Load files
    default_xsd_path = finders.find(
        main_registry_settings.REGISTRY_XSD_FILEPATH
    )
    list_xslt_path = finders.find(
        join(
            main_registry_settings.XSL_FOLDER_PATH,
            main_registry_settings.LIST_XSL_FILENAME,
        )
    )
    detail_xslt_path = finders.find(
        join(
            main_registry_settings.XSL_FOLDER_PATH,
            main_registry_settings.DETAIL_XSL_FILENAME,
        )
    )
    default_json_path = finders.find(
        main_registry_settings.CUSTOM_REGISTRY_FILE_PATH
    )
    if (
        not default_xsd_path
        or not list_xslt_path
        or not detail_xslt_path
        or not default_json_path
    ):
        raise BadMigrationError(
            "A file required for loading data was not found."
        )
    # Read files
    xsd_data = read_file_content(default_xsd_path)
    with open(default_json_path, encoding="utf-8") as json_file:
        json_data = json.load(json_file)
    list_xsl_data = read_file_content(list_xslt_path)
    detail_xsl_data = read_file_content(detail_xslt_path)

    # Build list of custom resources from JSON file
    custom_resources = custom_resource_api.get_resources_from_dict(json_data)

    # Insert template
    tvm = registry_system_api.insert_registry_schema(
        main_registry_settings.REGISTRY_XSD_FILENAME, xsd_data
    )
    current_template = tvm.current_version
    # Insert refinements
    refinement.init_refinements(current_template)
    # Insert custom resources
    for custom_resource in custom_resources:
        custom_resource.template = current_template
        custom_resource.save()

    # Save list and detail XSLT
    list_xslt = xslt_transformation_api.upsert(
        XslTransformation(
            name=main_registry_settings.LIST_XSL_FILENAME,
            filename=main_registry_settings.LIST_XSL_FILENAME,
            content=list_xsl_data,
        )
    )
    detail_xslt = xslt_transformation_api.upsert(
        XslTransformation(
            name=main_registry_settings.DETAIL_XSL_FILENAME,
            filename=main_registry_settings.DETAIL_XSL_FILENAME,
            content=detail_xsl_data,
        )
    )

    # Bind XSLT to template
    template_xsl_rendering_api.add_or_delete(
        template=current_template,
        list_xslt=list_xslt,
        default_detail_xslt=detail_xslt,
        list_detail_xslt=[detail_xslt],
    )



""" Unit test views
"""
from unittest.mock import patch

from django.contrib.messages.storage.fallback import FallbackStorage
from django.test import RequestFactory, SimpleTestCase

from core_main_app.commons.exceptions import ModelError
from core_main_app.components.template.models import Template
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_registry_app.views.admin import views as admin_views


class TestCustomRegistry(SimpleTestCase):
    """Test Custom Registry"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1", is_staff=True)

    @patch(
        "core_main_registry_app.components.template.api.get_current_registry_template"
    )
    @patch("core_main_registry_app.views.admin.views.admin_render")
    @patch.object(admin_views, "messages")
    def test_get_custom_registry_returns_rendered_page(
        self,
        mock_messages,
        mock_admin_render,
        mock_get_current_registry_template,
    ):
        """test_get_custom_registry_returns_rendered_page

        Returns:

        """
        # Arrange
        request = self.factory.get("core_main_registry_app_custom_registry")
        request.user = self.user1
        setattr(request, "session", "session")
        messages = FallbackStorage(request)
        setattr(request, "_messages", messages)

        # Act
        response = admin_views.CustomRegistry.as_view()(request)

        # Assert
        self.assertTrue(response, mock_admin_render.return_value)
        self.assertTrue(mock_get_current_registry_template.called)

    @patch(
        "core_main_registry_app.components.template.api.get_current_registry_template"
    )
    @patch("core_main_registry_app.views.admin.views.admin_render")
    @patch.object(admin_views, "messages")
    def test_get_custom_registry_with_model_error_returns_rendered_page_with_message(
        self,
        mock_messages,
        mock_admin_render,
        mock_get_current_registry_template,
    ):
        """test_get_custom_registry_returns_rendered_page

        Returns:

        """
        # Arrange
        mock_get_current_registry_template.side_effect = ModelError("error")
        request = self.factory.get("core_main_registry_app_custom_registry")
        request.user = self.user1
        setattr(request, "session", "session")
        messages = FallbackStorage(request)
        setattr(request, "_messages", messages)

        # Act
        response = admin_views.CustomRegistry.as_view()(request)

        # Assert
        self.assertTrue(response, mock_admin_render.return_value)
        self.assertTrue(mock_get_current_registry_template.called)
        self.assertTrue(mock_messages.add_message.called)


class TestManageTemplates(SimpleTestCase):
    """Test Manage Templates"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1", is_staff=True)

    @patch("core_main_registry_app.views.admin.views.admin_render")
    @patch("core_main_app.components.template_version_manager.api.get_by_id")
    @patch(
        "core_main_app.components.template_version_manager.api.get_global_version_managers"
    )
    @patch(
        "core_main_registry_app.components.template.api.get_current_registry_template"
    )
    def test_get_manage_templates_returns_rendered_page(
        self,
        mock_get_current_registry_template,
        mock_get_global_version_managers,
        mock_get_by_id,
        mock_admin_render,
    ):
        """test_get_manage_templates_returns_rendered_page

        Returns:

        """
        # Arrange
        request = self.factory.get("core_main_registry_app_templates")
        request.user = self.user1

        mock_get_current_registry_template.return_value = Template(
            version_manager_id=1
        )
        mock_get_global_version_managers.return_value = []
        mock_get_by_id.return_value = None
        # Act
        response = admin_views.ManageTemplatesRegistryView.as_view()(request)

        # Assert
        self.assertTrue(response, mock_admin_render.return_value)
        self.assertTrue(mock_get_current_registry_template.called)
        self.assertTrue(mock_get_global_version_managers.called)
        self.assertTrue(mock_get_by_id.called)



""" Unit Test for Custom Resource API
"""
from unittest import mock

from django.conf import settings
from django.test import TestCase
from tests.components.custom_resource.fixtures.fixtures import (
    CustomResourceFixtures,
)

from core_main_app.commons import exceptions as exceptions
from core_main_app.components.template.models import Template
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import create_mock_request
from core_main_registry_app.components.custom_resource import (
    api as custom_resource_api,
)
from core_main_registry_app.components.custom_resource.models import (
    CustomResource,
)
from core_main_registry_app.constants import CUSTOM_RESOURCE_TYPE

fixtureCustomResource = CustomResourceFixtures()


class TestCreateAllResourceCustomResource(TestCase):
    """
    Test Create All Resource Custom Resource
    """

    fixture = fixtureCustomResource

    def test_create_all_resource_custom_resource_return_type_all(self):
        """test_create_all_resource_custom_resource_return_type_all"""
        # Act
        key = "all_resource"
        custom_resource = custom_resource_api._create_custom_resource(
            self.fixture.create_custom_resource(),
            self.fixture.get_dict_custom_resource_all_resource()[key],
            key,
        )
        # Assert
        self.assertEqual(custom_resource.type, CUSTOM_RESOURCE_TYPE.ALL.value)

    def test_create_all_resource_custom_resource_return_icon(self):
        """test_create_all_resource_custom_resource_return_icon"""
        # Act
        key = "all_resource"
        custom_resource = custom_resource_api._create_custom_resource(
            self.fixture.create_custom_resource(),
            self.fixture.get_dict_custom_resource_all_resource()[key],
            key,
        )
        # Assert
        self.assertEqual(custom_resource.icon, "fa-globe")

    def test_create_all_resource_custom_resource_return_icon_color(self):
        """test_create_all_resource_custom_resource_return_icon_color"""
        # Act
        key = "all_resource"
        custom_resource = custom_resource_api._create_custom_resource(
            self.fixture.create_custom_resource(),
            self.fixture.get_dict_custom_resource_all_resource()[key],
            key,
        )
        # Assert
        self.assertEqual(custom_resource.icon_color, "#557EB9")

    def test_create_all_resource_custom_resource_return_sort(self):
        """test_create_all_resource_custom_resource_return_sort"""
        # Act
        key = "all_resource"
        custom_resource = custom_resource_api._create_custom_resource(
            self.fixture.create_custom_resource(),
            self.fixture.get_dict_custom_resource_all_resource()[key],
            key,
        )
        # Assert
        self.assertEqual(custom_resource.sort, 0)


class TestCreateCustomResource(TestCase):
    """
    Test Create Custom Resource
    """

    fixture = fixtureCustomResource

    def test_create_custom_resource_return_type_resource(self):
        """test_create_custom_resource_return_type_resource"""
        # Act
        key = "Organization"
        custom_resource = custom_resource_api._create_custom_resource(
            self.fixture.create_custom_resource(),
            self.fixture.get_dict_custom_resource()[key],
            key,
        )
        # Assert
        self.assertEqual(
            custom_resource.type, CUSTOM_RESOURCE_TYPE.RESOURCE.value
        )

    def test_create_custom_resource_return_name_in_schema(self):
        """test_create_custom_resource_return_name_in_schema"""
        # Act
        key = "Organization"
        custom_resource = custom_resource_api._create_custom_resource(
            self.fixture.create_custom_resource(),
            self.fixture.get_dict_custom_resource()[key],
            key,
        )
        # Assert
        self.assertEqual(custom_resource.name_in_schema, key)

    def test_create_custom_resource_return_title(self):
        """test_create_custom_resource_return_title"""
        # Act
        key = "Organization"
        custom_resource = custom_resource_api._create_custom_resource(
            self.fixture.create_custom_resource(),
            self.fixture.get_dict_custom_resource()[key],
            key,
        )
        # Assert
        self.assertEqual(custom_resource.title, "Organization")

    def test_create_custom_resource_return_description(self):
        """test_create_custom_resource_return_description"""

        # Act
        key = "Organization"
        custom_resource = custom_resource_api._create_custom_resource(
            self.fixture.create_custom_resource(),
            self.fixture.get_dict_custom_resource()[key],
            key,
        )
        # Assert
        self.assertEqual(
            custom_resource.description,
            "a group of people that come together to contribute to or "
            "participate in a federated data operation. Organizations can "
            "be hierarchical: an organization can contain or sponsor other "
            "organizations. Organizations can also aggregate or participate "
            "with other organizations in broader collaborations.",
        )

    def test_create_custom_resource_return_icon(self):
        """test_create_custom_resource_return_icon"""

        # Act
        key = "Organization"
        custom_resource = custom_resource_api._create_custom_resource(
            self.fixture.create_custom_resource(),
            self.fixture.get_dict_custom_resource()[key],
            key,
        )
        # Assert
        self.assertEqual(custom_resource.icon, "fa-university")

    def test_create_custom_resource_return_icon_color(self):
        """test_create_custom_resource_return_icon_color"""

        # Act
        key = "Organization"
        custom_resource = custom_resource_api._create_custom_resource(
            self.fixture.create_custom_resource(),
            self.fixture.get_dict_custom_resource()[key],
            key,
        )
        # Assert
        self.assertEqual(custom_resource.icon_color, "#2CAAE2")

    def test_create_custom_resource_return_display_icon(self):
        """test_create_custom_resource_return_display_icon"""

        # Act
        key = "Organization"
        custom_resource = custom_resource_api._create_custom_resource(
            self.fixture.create_custom_resource(),
            self.fixture.get_dict_custom_resource()[key],
            key,
        )
        # Assert
        self.assertTrue(custom_resource.display_icon)

    def test_create_custom_resource_return_role_choice(self):
        """test_create_custom_resource_return_role_choice"""

        # Act
        key = "Organization"
        custom_resource = custom_resource_api._create_custom_resource(
            self.fixture.create_custom_resource(),
            self.fixture.get_dict_custom_resource()[key],
            key,
        )
        # Assert
        self.assertEqual(custom_resource.role_choice, "Organization")

    def test_create_custom_resource_return_role_type(self):
        """test_create_custom_resource_return_role_type"""

        # Act
        key = "Organization"
        custom_resource = custom_resource_api._create_custom_resource(
            self.fixture.create_custom_resource(),
            self.fixture.get_dict_custom_resource()[key],
            key,
        )
        # Assert
        self.assertEqual(custom_resource.role_type, "Organization")

    def test_create_custom_resource_return_sort(self):
        """test_create_custom_resource_return_sort"""

        # Act
        key = "Organization"
        custom_resource = custom_resource_api._create_custom_resource(
            self.fixture.create_custom_resource(),
            self.fixture.get_dict_custom_resource()[key],
            key,
        )
        # Assert
        self.assertEqual(custom_resource.sort, 1)


class TestIsCustomResourceTypeResource(TestCase):
    """
    Test Is Custom Resource Type Resource
    """

    fixture = fixtureCustomResource

    def test_is_all_resource_return_type_resource(self):
        """test_is_all_resource_return_type_resource"""

        # Act
        custom_resource = CustomResource(
            type=CUSTOM_RESOURCE_TYPE.RESOURCE.value
        )
        # Assert
        self.assertTrue(
            custom_resource_api._is_custom_resource_type_resource(
                custom_resource
            )
        )

    def test_is_all_resource_return_type_all(self):
        """test_is_all_resource_return_type_all"""

        # Act
        custom_resource = CustomResource(type=CUSTOM_RESOURCE_TYPE.ALL.value)
        # Assert
        self.assertFalse(
            custom_resource_api._is_custom_resource_type_resource(
                custom_resource
            )
        )


class TestIsTypeAll(TestCase):
    """
    Test Is Type All
    """

    fixture = fixtureCustomResource

    def test_is_all_resource_return_is_resource_custom_resource(self):
        """test_is_all_resource_return_is_resource_custom_resource"""

        # Act
        key = "Organization"
        resource = self.fixture.get_dict_custom_resource()[key]
        # Assert
        self.assertFalse(custom_resource_api._is_type_all(resource))

    def test_is_all_resource_return_is_resource_all_resource(self):
        """test_is_all_resource_return_is_resource_all_resource"""

        # Act
        key = "all_resource"
        resource = self.fixture.get_dict_custom_resource_all_resource()[key]
        # Assert
        self.assertTrue(custom_resource_api._is_type_all(resource))

    def test_is_all_resource_raise_exception(self):
        """test_is_all_resource_raise_exception"""

        # Act
        resource = self.fixture.get_dict_custom_resource_no_type()
        # Assert
        with self.assertRaises(exceptions.ModelError):
            custom_resource_api._is_type_all(resource)


class TestCheckCurate(TestCase):
    """
    Test Check Curate
    """

    fixture = fixtureCustomResource

    def test_check_curate_core_curate_app_not_installed_return_none(self):
        """test_check_curate_core_curate_app_not_installed_return_none"""

        # Act
        custom_resource = CustomResource()
        # Assert
        self.assertIsNone(custom_resource_api._check_curate(custom_resource))

    def test_check_curate_is_not_resource_return_none(self):
        """test_check_curate_is_not_resource_return_none"""

        # Act
        custom_resource = CustomResource()
        # Assert
        with mock.patch.object(
            settings, "INSTALLED_APPS", ["core_curate_app"]
        ):
            self.assertIsNone(
                custom_resource_api._check_curate(custom_resource)
            )

    def test_check_curate_is_resource_return_except(self):
        """test_check_curate_is_resource_return_except"""

        # Act
        custom_resource = CustomResource(
            type=CUSTOM_RESOURCE_TYPE.RESOURCE.value
        )
        # Assert
        with mock.patch.object(
            settings, "INSTALLED_APPS", ["core_curate_app"]
        ):
            with self.assertRaises(exceptions.ModelError):
                custom_resource_api._check_curate(custom_resource)

    def test_check_curate_is_resource_with_role_type_return_except(self):
        """test_check_curate_is_resource_with_role_type_return_except"""

        # Act
        custom_resource = CustomResource(
            type=CUSTOM_RESOURCE_TYPE.RESOURCE.value, role_type="role_type"
        )
        # Assert
        with mock.patch.object(
            settings, "INSTALLED_APPS", ["core_curate_app"]
        ):
            with self.assertRaises(exceptions.ModelError):
                custom_resource_api._check_curate(custom_resource)

    def test_check_curate_is_resource_with_role_choice_return_except(self):
        """test_check_curate_is_resource_with_role_choice_return_except"""

        # Act
        custom_resource = CustomResource(
            type=CUSTOM_RESOURCE_TYPE.RESOURCE.value, role_choice="role_choice"
        )
        # Assert
        with mock.patch.object(
            settings, "INSTALLED_APPS", ["core_curate_app"]
        ):
            with self.assertRaises(exceptions.ModelError):
                custom_resource_api._check_curate(custom_resource)

    def test_check_curate_is_resource_with_all_return_none(self):
        """test_check_curate_is_resource_with_all_return_none"""

        # Act
        custom_resource = CustomResource(
            type=CUSTOM_RESOURCE_TYPE.RESOURCE.value,
            role_type="role_type",
            role_choice="role_choice",
        )
        # Assert
        with mock.patch.object(
            settings, "INSTALLED_APPS", ["core_curate_app"]
        ):
            self.assertIsNone(
                custom_resource_api._check_curate(custom_resource)
            )


class TestGetByCurrentTemplateAndSlug(TestCase):
    """
    Test Get By Current Template And Slug
    """

    @mock.patch.object(
        CustomResource, "get_custom_resource_by_template_and_slug"
    )
    @mock.patch(
        "core_main_registry_app.components.custom_resource.api._get_current_template"
    )
    def test_get_returns_custom_resource(
        self, get_current, get_custom_resource_by_template_and_slug
    ):
        """test_get_returns_custom_resource"""

        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(user=mock_user)
        get_custom_resource_by_template_and_slug.return_value = (
            CustomResource()
        )
        get_current.return_value = Template()
        # Assert
        self.assertTrue(
            isinstance(
                custom_resource_api.get_by_current_template_and_slug(
                    "test", request=mock_request
                ),
                CustomResource,
            )
        )

    @mock.patch.object(
        CustomResource, "get_custom_resource_by_template_and_slug"
    )
    def test_get_absent_slug_or_template_raises_does_not_exist(
        self, get_custom_resource_by_template_and_slug
    ):
        """test_get_absent_slug_or_template_raises_DoesNotExist"""

        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(user=mock_user)
        get_custom_resource_by_template_and_slug.side_effect = (
            exceptions.DoesNotExist("error")
        )
        # Assert
        with self.assertRaises(exceptions.DoesNotExist):
            custom_resource_api.get_by_current_template_and_slug(
                "test", request=mock_request
            )



"""
Constants for core main registry app
"""
from enum import Enum

# Create an unspecified category when 2+ levels.
UNSPECIFIED_CATEGORY = True
# Path to status
PATH_STATUS = "Resource.@status"

CUSTOM_RESOURCE_TYPE = Enum(
    "CUSTOM_RESOURCE_TYPE", {"RESOURCE": "resource", "ALL": "all"}
)

CATEGORY_SUFFIX = "__category"
UNSPECIFIED_LABEL = "unspecified"



""" System api to access data without access control neither API rules
"""
from django.db.models import Q

from core_main_app.commons import exceptions as exceptions
from core_main_app.components.data.models import Data
from core_main_app.components.template.models import Template
from core_main_app.components.template_version_manager.models import (
    TemplateVersionManager,
)
from core_main_app.components.version_manager.utils import (
    get_latest_version_name,
)
from core_main_app.settings import MONGODB_INDEXING
from core_main_app.system import api as main_system_api
from core_main_app.utils.xml import is_schema_valid, get_hash
from core_main_registry_app.settings import REGISTRY_XSD_FILENAME


def is_local_id_already_used(local_id):
    """Check if the local id given already exist in db

    Args:
        local_id:

    Returns:

    """
    if MONGODB_INDEXING:
        from core_main_app.components.mongo.models import MongoData

        return MongoData.objects(
            __raw__={"dict_content.Resource.@localid": str(local_id)}
        )
    return Data.objects.filter(
        Q(**{"dict_content__Resource__@localid": str(local_id)})
    )


def insert_registry_schema(xsd_filename, xsd_content):
    """insert registry schema.

    Args:
        xsd_filename:
        xsd_content:

    Returns:

    """
    # Check if schema is valid
    is_schema_valid(xsd_content)
    template = Template(
        filename=xsd_filename, hash=get_hash(xsd_content), content=xsd_content
    )
    template.save_template()
    # save the template in database
    try:
        template_version_manager = TemplateVersionManager(title=xsd_filename)
        template_version_manager.save_version_manager()
        # insert the initial template in the version manager
        template.version_manager = template_version_manager
        # set current version
        if len(template_version_manager.versions) == 0:
            template.is_current = True
        # update saved template
        template.display_name = get_latest_version_name(
            template_version_manager
        )
        # save template
        template.save()
        # return version manager
        return template_version_manager
    except Exception as exception:
        template.delete()
        raise exception


def get_current_registry_template():
    """Get the current template used for the registry.

    Args:

    Returns:

    """
    try:
        template_version = (
            main_system_api.get_active_global_version_manager_by_title(
                REGISTRY_XSD_FILENAME
            )
        )
        return main_system_api.get_template_by_id(template_version.current)
    except Exception as exception:
        raise exceptions.ModelError(str(exception))



""" Unit Test Data
"""
from unittest.mock import patch


from core_main_app.commons import exceptions as exceptions
from core_main_app.components.data.models import Data
import core_main_app.components.workspace.api as workspace_api
from core_main_app.utils.integration_tests.integration_base_test_case import (
    IntegrationBaseTestCase,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import create_mock_request

import core_main_registry_app.components.data.api as data_registry_api
from core_main_registry_app.commons.constants import DataStatus
from tests.components.data.fixtures.fixtures import DataRegistryFixtures


fixture_data = DataRegistryFixtures()


class TestDataGetStatus(IntegrationBaseTestCase):
    """Test Data Get Status"""

    fixture = fixture_data

    def test_data_get_status_return_status_from_xml_content(self):
        """test_data_get_status_return_status_from_xml_content"""

        # Act
        status = data_registry_api.get_status(self.fixture.data_1)
        # Assert
        self.assertTrue(status == DataStatus.ACTIVE)

    def test_data_get_status_returns_none_if_status_key_does_not_exist(
        self,
    ):
        """test_data_get_status_returns_none_if_status_key_does_not_exist"""

        # Act
        status = data_registry_api.get_status(self.fixture.data_2)
        # Assert
        self.assertIsNone(status)


class TestDataSetStatus(IntegrationBaseTestCase):
    """Test Data Set Status"""

    fixture = fixture_data

    @patch.object(Data, "convert_to_file")
    def test_data_set_status_to_inactive_should_always_work(
        self, mock_convert_to_file
    ):
        """test_data_set_status_to_inactive_should_always_work"""

        # Arrange
        user = create_mock_user("1", True, True)
        mock_request = create_mock_request(user=user)
        status = data_registry_api.get_status(self.fixture.data_1)
        self.assertTrue(status == DataStatus.ACTIVE)
        # Act
        data = data_registry_api.set_status(
            self.fixture.data_1, DataStatus.INACTIVE, mock_request
        )
        status = data_registry_api.get_status(data)
        # Assert
        self.assertTrue(status == DataStatus.INACTIVE)

    @patch.object(Data, "convert_to_file")
    def test_data_set_status_to_deleted_raise_exception_if_data_is_not_published(
        self, mock_convert_to_file
    ):
        """test_data_set_status_to_deleted_raise_exception_if_data_is_not_published"""

        # Arrange
        user = create_mock_user("1", True, True)
        mock_request = create_mock_request(user=user)
        status = data_registry_api.get_status(self.fixture.data_1)
        self.assertTrue(status == DataStatus.ACTIVE)
        # Act Assert
        with self.assertRaises(exceptions.ModelError):
            data_registry_api.set_status(
                self.fixture.data_1, DataStatus.DELETED, mock_request
            )


class TestDataPublish(IntegrationBaseTestCase):
    """Test Data Publish"""

    fixture = fixture_data

    @patch.object(workspace_api, "get_global_workspace")
    def test_publish_data_without_workspace_publishes_data(
        self, get_global_workspace
    ):
        """test_publish_data_without_workspace_publishes_data"""

        # Arrange
        user = create_mock_user("1", is_superuser=True, has_perm=True)
        get_global_workspace.return_value = self.fixture.workspace_1

        # Act
        data_registry_api.publish(self.fixture.data_1, user)

        # Assert
        self.assertEqual(
            self.fixture.data_1.workspace, get_global_workspace.return_value
        )

    @patch.object(workspace_api, "get_global_workspace")
    def test_publish_data_in_private_workspace_publishes_data(
        self, get_global_workspace
    ):
        """test_publish_data_in_private_workspace_publishes_data"""

        # Arrange
        user = create_mock_user("1", is_superuser=True, has_perm=True)
        get_global_workspace.return_value = self.fixture.workspace_1

        # Act
        data_registry_api.publish(self.fixture.data_2, user)

        # Assert
        self.assertEqual(
            self.fixture.data_2.workspace, get_global_workspace.return_value
        )

    @patch.object(workspace_api, "get_global_workspace")
    def test_publish_data_in_public_raises_error(self, get_global_workspace):
        """test_publish_data_in_public_raises_error"""

        # Arrange
        user = create_mock_user("1", is_superuser=True, has_perm=True)
        get_global_workspace.return_value = self.fixture.workspace_1
        self.fixture.data_2.workspace = self.fixture.workspace_1

        # Act Assert
        with self.assertRaises(exceptions.ApiError):
            data_registry_api.publish(self.fixture.data_2, user)



""" Role extraction API
"""


def role_extraction(dict_content):
    """Extract role from dict content.

    Args:
        dict_content

    Returns:
    """
    try:
        list_resource_role = dict_content["Resource"]["role"]
        list_role = []
        if "@xsi:type" in list_resource_role:
            list_role.append(list_resource_role["@xsi:type"])
            return list_role
        for dict_role in list_resource_role:
            list_role.append(dict_role["@xsi:type"])
        return list_role
    except KeyError:
        return None



""" Authentication tests for Data REST API
"""
from django.test import SimpleTestCase
from unittest.mock import patch, MagicMock
from rest_framework import status

from core_main_app.components.data import api as data_api
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock
from core_main_registry_app.rest.data import views as data_rest_views


class TestDataDetailDeletePermissions(SimpleTestCase):
    """Test Data Detail Delete Permissions"""

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403"""

        response = RequestMock.do_request_delete(
            data_rest_views.DataDetailRegistry.as_view(), None, param={"pk": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(
        self,
    ):
        """test_authenticated_returns_http_403"""

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_delete(
            data_rest_views.DataDetailRegistry.as_view(),
            mock_user,
            param={"pk": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_staff_returns_http_403(
        self,
    ):
        """test_staff_returns_http_403"""

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_delete(
            data_rest_views.DataDetailRegistry.as_view(),
            mock_user,
            param={"pk": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_superuser_returns_http_403(
        self,
    ):
        """test_superuser_returns_http_403"""

        mock_user = create_mock_user("1", is_superuser=True)

        response = RequestMock.do_request_delete(
            data_rest_views.DataDetailRegistry.as_view(),
            mock_user,
            param={"pk": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(data_api, "delete")
    @patch.object(data_api, "get_by_id")
    def test_superuser_and_staff_returns_http_204(
        self, mock_data_api_get_by_id, mock_data_api_delete
    ):
        """test_superuser_and_staff_returns_http_204"""

        mock_data_api_get_by_id.return_value = None
        mock_data_api_delete.return_value = None

        mock_user = create_mock_user("1", is_staff=True, is_superuser=True)

        response = RequestMock.do_request_delete(
            data_rest_views.DataDetailRegistry.as_view(),
            mock_user,
            param={"pk": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)
        self.assertTrue(mock_data_api_delete.called)


class TestDataDetailPatchPermissions(SimpleTestCase):
    """Test Data Detail Patch Permissions"""

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403"""

        response = RequestMock.do_request_patch(
            data_rest_views.DataDetailRegistry.as_view(),
            None,
            data={"title": "edited"},
            param={"pk": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(
        self,
    ):
        """test_authenticated_returns_http_403"""
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_patch(
            data_rest_views.DataDetailRegistry.as_view(),
            mock_user,
            data={"title": "edited"},
            param={"pk": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_staff_returns_http_403(
        self,
    ):
        """test_staff_returns_http_403"""
        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_patch(
            data_rest_views.DataDetailRegistry.as_view(),
            mock_user,
            data={"title": "edited"},
            param={"pk": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_superuser_returns_http_403(
        self,
    ):
        """test_superuser_returns_http_403"""
        mock_user = create_mock_user("1", is_superuser=True)

        response = RequestMock.do_request_patch(
            data_rest_views.DataDetailRegistry.as_view(),
            mock_user,
            data={"title": "edited"},
            param={"pk": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(data_api, "get_by_id")
    @patch.object(data_api, "upsert")
    def test_superuser_and_staff_returns_http_200(
        self, mock_data_api_get_by_id, mock_data_api_upsert
    ):
        """test_superuser_and_staff_returns_http_200"""

        mock_data = MagicMock(title="title")
        mock_data_api_get_by_id.return_value = mock_data
        mock_data_api_upsert.return_value = mock_data

        mock_user = create_mock_user("1", is_staff=True, is_superuser=True)

        response = RequestMock.do_request_patch(
            data_rest_views.DataDetailRegistry.as_view(),
            mock_user,
            data={"title": "edited"},
            param={"pk": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertTrue(mock_data_api_upsert.called)
